import os
import time
from collections import namedtuple
from .. import exceptions
try:
    from binstar_client.utils import get_server_api
    from binstar_client import errors
except ImportError:
    get_server_api = None


ENVIRONMENT_TYPE = 'env'

# TODO: refactor binstar so that individual arguments are passed in instead of an arg object
binstar_args = namedtuple('binstar_args', ['site', 'token'])


def is_installed():
    """
    is Binstar-cli installed?
    :return: True/False
    """
    return get_server_api is not None


class Uploader(object):
    """
    Upload environments to Binstar
    * Check if user is logged (offer to log in if it's not)
    * Check if package exist (create if not)
    * Check if distribution exist (overwrite if force=True)
    * Upload environment.yml
    """

    _user = None
    _username = None
    _binstar = None

    def __init__(self, packagename, env_file, **kwargs):
        self.packagename = packagename
        self.file = env_file
        self.summary = kwargs.get('summary')
        self.env_data = kwargs.get('env_data')
        self.basename = os.path.basename(env_file)

    @property
    def version(self):
        return time.strftime('%Y.%m.%d.%H%M')

    @property
    def user(self):
        if self._user is None:
            self._user = self.binstar.user()
        return self._user

    @property
    def binstar(self):
        if self._binstar is None:
            self._binstar = get_server_api()
        return self._binstar

    @property
    def username(self):
        if self._username is None:
            self._username = self.user['login']
        return self._username

    def authorized(self):
        try:
            return self.user is not None
        except errors.Unauthorized:
            return False

    def upload(self):
        """
        Prepares and uploads env file
        :return: True/False
        """
        print("Uploading environment %s to anaconda-server (%s)... " %
              (self.packagename, self.binstar.domain))
        if self.is_ready():
            with open(self.file, mode='rb') as envfile:
                return self.binstar.upload(self.username, self.packagename,
                                           self.version, self.basename, envfile,
                                           distribution_type=ENVIRONMENT_TYPE, attrs=self.env_data)
        else:
            raise exceptions.AlreadyExist()

    def is_ready(self):
        """
        Ensures package namespace and distribution
        :return: True or False
        """
        return self.ensure_package_namespace() and self.ensure_distribution()

    def ensure_package_namespace(self):
        """
        Ensure that a package namespace exists. This is required to upload a file.
        """
        try:
            self.binstar.package(self.username, self.packagename)
        except errors.NotFound:
            self.binstar.add_package(self.username, self.packagename, self.summary)

        # TODO: this should be removed as a hard requirement of binstar
        try:
            self.binstar.release(self.username, self.packagename, self.version)
        except errors.NotFound:
            self.binstar.add_release(self.username, self.packagename, self.version, {}, '', '')

        return True

    def ensure_distribution(self):
        """
        Ensure that a package distribution does not exist.
        """
        try:
            self.binstar.distribution(self.username, self.packagename, self.version, self.basename)
        except errors.NotFound:
            return True
        else:
            return False



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals
from logging import getLogger
import os
from os.path import join, isdir, lexists, isfile, exists
from tempfile import gettempdir
from unittest import TestCase
import uuid

from conda.common.compat import on_win, PY2

from conda.gateways.disk.create import mkdir_p
from conda.gateways.disk.delete import rm_rf
from conda.gateways.disk.link import link, islink, readlink, stat_nlink, symlink
from conda.gateways.disk.update import touch

log = getLogger(__name__)


class LinkSymlinkUnlinkIslinkReadlinkTests(TestCase):

    def setUp(self):
        tempdirdir = gettempdir()
        dirname = str(uuid.uuid4())[:8]
        self.test_dir = join(tempdirdir, dirname)
        mkdir_p(self.test_dir)
        assert isdir(self.test_dir)

    def tearDown(self):
        rm_rf(self.test_dir)
        assert not lexists(self.test_dir)

    def test_hard_link(self):
        path1_real_file = join(self.test_dir, 'path1_real_file')
        path2_second_inode = join(self.test_dir, 'path2_second_inode')
        touch(path1_real_file)
        assert isfile(path1_real_file)
        assert not islink(path1_real_file)
        link(path1_real_file, path2_second_inode)
        assert isfile(path2_second_inode)
        assert not islink(path2_second_inode)

        path1_stat = os.stat(path1_real_file)
        path2_stat = os.stat(path2_second_inode)

        assert path1_stat.st_ino == path2_stat.st_ino
        assert stat_nlink(path1_real_file) == stat_nlink(path2_second_inode)

        os.unlink(path2_second_inode)
        assert not lexists(path2_second_inode)
        assert stat_nlink(path1_real_file) == 1

        os.unlink(path1_real_file)
        assert not lexists(path1_real_file)

    def test_soft_link(self):
        path1_real_file = join(self.test_dir, 'path1_real_file')
        path2_symlink = join(self.test_dir, 'path2_symlink')
        touch(path1_real_file)
        assert isfile(path1_real_file)
        assert not islink(path1_real_file)

        symlink(path1_real_file, path2_symlink)
        assert exists(path2_symlink)
        assert lexists(path2_symlink)
        assert islink(path2_symlink)

        assert readlink(path2_symlink).endswith(path1_real_file)
        # for win py27, readlink actually gives something that starts with \??\
        # \??\c:\users\appveyor\appdata\local\temp\1\c571cb0c\path1_real_file

        assert stat_nlink(path1_real_file) == stat_nlink(path2_symlink) == 1

        os.unlink(path1_real_file)
        assert not isfile(path1_real_file)
        assert not lexists(path1_real_file)
        assert not exists(path1_real_file)

        assert lexists(path2_symlink)
        if not (on_win and PY2):
            # I guess I'm not surprised this exist vs lexist is different for win py2
            #   consider adding a fix in the future
            assert not exists(path2_symlink)

        os.unlink(path2_symlink)
        assert not lexists(path2_symlink)
        assert not exists(path2_symlink)



import json
import unittest
from conda_env.utils.notebooks import Notebook
from ..utils import support_file

notebook = {
    "metadata": {
        "name": "",
        "signature": ""
    },
    "nbformat": 3,
    "nbformat_minor": 0,
    "worksheets": [
        {
            "cells": []
        }
    ]
}


class NotebookTestCase(unittest.TestCase):
    def test_notebook_not_exist(self):
        nb = Notebook('no-exist.ipynb')
        self.assertEqual(nb.inject('content'), False)
        self.assertEqual(nb.msg, "no-exist.ipynb may not exist or you don't have adequate permissions")

    def test_environment_already_exist(self):
        nb = Notebook(support_file('notebook-with-env.ipynb'))
        self.assertEqual(nb.inject('content'), False)

    def test_inject_env(self):
        nb = Notebook(support_file('notebook.ipynb'))
        self.assertTrue(nb.inject('content'))

        with open(support_file('notebook.ipynb'), 'w') as fb:
            fb.write(json.dumps(notebook, sort_keys=True))

    def test_inject(self):
        nb = Notebook(support_file('notebook.ipynb'))
        self.assertTrue(nb.inject('user/environment'))

        with open(support_file('notebook.ipynb'), 'w') as fb:
            fb.write(json.dumps(notebook, sort_keys=True))
